# -*- coding: latin-1 -*-


import sys
sys.path.insert(0, '..')

from pynotebook.nbview import TextModel, NBView, ScriptingCell
from pynotebook.textformat import fromtext
import wx



examples = r"""[Text]:
Matplotlib Demo

The notebook consists of cells: scripting cells and text cells. The python code in scripting cells can be executed by moving the cursor in the cell and pressing shift + return. Scripting cells have an input field (marked by "In") and an output field ("Out"). Output usually consists of text printed to stdout or stderr by the python interpreter. Pynotbook can also display matplotlib graphics, other output types can be easily implemented. This demo shows how to output graphics created by matplotlib.

Try to execute the following cells.

Notes:
 - Use the tab-key to complete. 
 - To create a new cell, place the cursor below any output cell and start typing. 
 - You can copy, paste and delete cells.
 - There is undo (ctrl-z) and redo (ctrl-r).

Demo codes are taken from 
http://www.southampton.ac.uk/~fangohr/training/python/notebooks/Matplotlib.html


[In 0]:
import matplotlib
matplotlib.use('Agg')
from pylab import *

if matplotlib.__version__ >= '1.4':
    matplotlib.style.use('ggplot')


[In 0]:
fig = plt.figure(facecolor='white')
r = np.arange(0, 3.0, 0.01)
theta = 2 * np.pi * r

ax = plt.subplot(111, polar=True)
ax.plot(theta, r, color='r', linewidth=3)
ax.set_rmax(2.0)
ax.grid(True)

ax.set_title("A line plot on a polar axis", va='bottom')
output(fig)

[In 0]:
fig # ... and again, this time without an explicit call to "output"!
[In 0]:
from numpy import cos
alpha = 0.7
phi_ext = 2 * pi * 0.5

def flux_qubit_potential(phi_m, phi_p):
    return 2 + alpha - 2 * cos(phi_p)*cos(phi_m) - alpha * cos(phi_ext - 2*phi_p)

phi_m = linspace(0, 2*pi, 100)
phi_p = linspace(0, 2*pi, 100)
X,Y = meshgrid(phi_p, phi_m)
Z = flux_qubit_potential(X, Y).T

fig, ax = plt.subplots()
fig.set_facecolor('white')

p = ax.pcolor(X/(2*pi), Y/(2*pi), Z, cmap=cm.RdBu, vmin=abs(Z).min(), vmax=abs(Z).max())
cb = fig.colorbar(p, ax=ax)
output(fig)
[In 0]:
fig, ax = plt.subplots(facecolor='white')
cnt = contour(Z, cmap=cm.RdBu, vmin=abs(Z).min(), vmax=abs(Z).max(), extent=[0, 1, 0, 1])
output(fig)

[In]:
from mpl_toolkits.mplot3d import axes3d, Axes3D
fig = plt.figure()
ax = Axes3D(fig)
X, Y, Z = axes3d.get_test_data(0.05)
cset = ax.contour(X, Y, Z, 16, extend3d=True)
ax.clabel(cset, fontsize=9, inline=1)
output(fig)

"""


def demo_00():
    app = wx.App(redirect=False)
    model = fromtext(examples)
    # Apply some formating to make the demo look better
    model.set_properties(0, 16, fontsize=16, weight='bold', underline=True)
    model.set_properties(0, 876, family='swiss' )
    model.set_properties(558, 563, underline=True)
    frame = wx.Frame(None)
    win = wx.Panel(frame, -1)
    view = NBView(win, -1, style=wx.SUNKEN_BORDER)
    view.model = model
    box = wx.BoxSizer(wx.VERTICAL)
    box.Add(view, 1, wx.ALL|wx.GROW, 1)
    win.SetSizer(box)
    win.SetAutoLayout(True)
    frame.Show()

    if 0:
        from pynotebook.wxtextview import testing
        testing.pyshell(locals())

    if 0:
        import celleinspector
        inspector = cellinspector.Inspector(frame)
        inspector.model = view
        inspector.Show()

    app.MainLoop()


if __name__ == '__main__':
    demo_00()



# -*- coding: latin-1 -*-

# Minimal model functionality needed for textmodel


#
#		Python GUI - Models - Generic
#

import weakref

#  List of views for a model is kept separately so that models
#  can be pickled without fear of accidentally trying to pickle
#  the user interface.

_model_views = weakref.WeakKeyDictionary() # {Model: [object]}

from .properties import overridable_property


class Model(object):
    """A Model represents an application object which can appear in a View.
    Each Model can have any number of Views attached to it. When a Model is
    changed, it should notify all of its Views so that they can update
    themselves."""

    views = overridable_property('views',
        "List of objects observing this model. Do not modify directly.")

    def destroy(self):
        """All views currently observing this model are removed, and their
        'model_destroyed' methods, if any, are called with the model as
        an argument."""
        self.notify_views('model_destroyed')
        for view in self.views[:]:
            self.remove_view(view)
            #self._call_if_present(view, 'model_destroyed', self)

    def get_views(self):
        views = _model_views.get(self)
        if views is None:
            views = []
            _model_views[self] = views
        return views

    #
    #		Adding and removing views
    #

    def add_view(self, view):
        """Add the given object as an observer of this model. The view will
        typically be a View subclass, but need not be. If the view is not
        already an observer of this model and defines an 'add_model' method,
        this method is called with the model as an argument."""
        views = self.views
        if view not in views:
            views.append(view)
            self._call_if_present(view, 'add_model', self)

    def remove_view(self, view):
        """If the given object is currently an observer of this model, it
        is removed, and if it defines a 'remove_model' method, this method
        is called with the model as an argument."""
        views = self.views
        if view in views:
            views.remove(view)
            self._call_if_present(view, 'remove_model', self)

    #
    #		View notification
    #

    def notify_views(self, message = 'model_changed', *args, **kwds):
        """For each observer, if the observer defines a method with
        the name of the message, call it with the given
        arguments. Otherwise, if it defines a method called
        'model_changed', call it with no arguments. Otherwise, do
        nothing for that observer."""
        for view in self.views:
            if not self._call_if_present(view, message, self, *args, **kwds):
                self._call_if_present(view, 'model_changed', self)

    def _call_if_present(self, obj, method_name, *args, **kwds):
        method = getattr(obj, method_name, None)
        if method:
            method(*args, **kwds)
            return 1
        else:
            return 0





# -*_ coding: latin-1 -*-

# The purpose of streams is to combine the stdout and stderr channels
# into a single channel, aka a "stream".


from .textmodel import TextModel
from .textmodel.texeltree import Texel


class StreamBase:
    # Stream protocol
    def output(self, obj, iserr=False):
        pass


class StreamRecorder(StreamBase):
    # A stream which stores all output calls into a list
    
    def __init__(self):
        self.messages = []

    def output(self, arg, iserr=False):
        self.messages.append((arg, iserr))



class Stream(StreamBase):
    # A stream which stores the output into a textmodel

    def __init__(self):
        self.model = TextModel()

    def output(self, obj, iserr=False):
        if isinstance(obj, Texel):
            new = TextModel()
            new.texel = obj
        else:
            if iserr:
                properties = {'textcolor':'red'}
            else:
                properties = {}
            if isinstance(obj, unicode):
                new = TextModel(obj, **properties)
            elif isinstance(obj, str):
                u = unicode(obj, 'utf-8')
                new = TextModel(u, **properties)
            else:
                new = TextModel(str(obj), **properties)
        self.model.append(new)



# -*- coding: latin-1 -*-


from ..textmodel import texeltree
from ..textmodel.textmodel import TextModel
from ..textmodel.texeltree import NewLine, Group, Text, length
from .testdevice import TESTDEVICE
from .boxes import TextBox, NewlineBox, TabulatorBox, EmptyTextBox, \
    EndBox, check_box, Box, calc_length



class Factory:
    TextBox = TextBox
    NewlineBox = NewlineBox
    TabulatorBox = TabulatorBox
    EndBox = EndBox

    def __init__(self, device=TESTDEVICE):
        self.device = device

    def get_device(self):
        return self.device

    ### Factory methods
    def create_all(self, texel):
        # Convenience method
        return self.create_boxes(texel, 0, length(texel))

    def create_boxes(self, texel, i1, i2):
        assert i1>=0
        assert i2<=length(texel)
        assert i1<=i2
        if i1 == i2:
            return () # XXX Why is this needed?
        name = texel.__class__.__name__+'_handler'
        handler = getattr(self, name)
        #print "calling handler", name, i1, i2
        l = handler(texel, i1, i2)
        try:
            assert calc_length(l) == i2-i1
        except:
            print "handler=", handler
            raise
        return tuple(l)
        
    def Group_handler(self, texel, i1, i2):
        r = []
        for j1, j2, child in texeltree.iter_childs(texel):
            if i1 < j2 and j1 < i2: # overlapp
                r.extend(self.create_boxes(
                    child, max(0, i1-j1), min(i2, j2)-j1))
        return r

    def Text_handler(self, texel, i1, i2):
        return [self.TextBox(texel.text[i1:i2], texel.style, self.device)]

    def NewLine_handler(self, texel, i1, i2):
        if texel.is_endmark:
            return [self.EndBox(texel.style, self.device)]
        return [self.NewlineBox(texel.style, self.device)] # XXX: Hmmmm

    def Tabulator_handler(self, texel, i1, i2):
        return [self.TabulatorBox(texel.style, self.device)]





class BuilderBase:
    """The builder is responsible for creating and updating the layout. 

    The methods build, insert, remove and update return a tree of boxes (=
    "layout").

    The length of the box tree is always the length of the model +1,
    because we add a special box ("end mark").

    """

    _layout = None

    def rebuild(self):
        # sets self._layout
        pass

    def get_layout(self):
        assert self._layout is not None
        return self._layout

    ### Signal handlers
    def properties_changed(self, i1, i2):
        pass

    def inserted(self, i, n):
        pass

    def removed(self, i, n):
        pass



def test_01():
    factory = Factory()
    boxes = factory.create_all(TextModel("123").texel)
    assert calc_length(boxes) == 3
    boxes = factory.create_all(TextModel("123\n567").get_xtexel())
    assert calc_length(boxes) == 8

    





# -*- coding: latin-1 -*-

from . import texeltree
from .texeltree import G, T, length, grouped, provides_childs, iter_childs, \
    is_root_efficient, is_list_efficient, is_homogeneous, calc_length, \
    get_pieces, fuse, EMPTYSTYLE, NL, NewLine


debug = 0


class StyleIterator:
    n = 0
    style = None
    total = 0
    finished = False
    def __init__(self, iterable):
        self.iterable = iterable
        self.advance(0)
        
    def advance(self, m):
        #print "advance", m
        self.total += m
        self.n -= m
        try:
            while self.n <= 0:
                n, self.style = self.iterable.next()
                self.n += n
        except StopIteration:
            self.finished = True



style_pool = {():EMPTYSTYLE}


def hash_style(style):
    return tuple(sorted(style.items()))


def create_style(**kwds):
    global style_pool
    key = hash_style(kwds)
    try:
        return style_pool[key]
    except KeyError:
        style_pool[key] = kwds
        return kwds


def updated_style(style, properties):
    new = style.copy()
    new.update(properties)
    return create_style(**new)


def get_style(texel, i):
    if i < 0 or i >= length(texel):
        raise IndexError(i)
    if provides_childs(texel):
        for i1,i2, child in iter_childs(texel):
            if i1 <= i < i2:
                return get_style(child, i-i1)
    return texel.style


def fuse_styles(l1, l2):
    if len(l1) and len(l2):
        tail = l1[-1]
        n, style = l2[0]
        if tail[1] is style:
            return l1[:-1]+[(tail[0]+n, style)]+l2[1:]
    return l1+l2


def get_styles(texel, i1, i2):
    """
    pre:
        0 <= i1 <= i2 <= length(texel)
    post:
        style_length(__return__) == i2-i1
    """
    if i1 == i2:
        return []
    if provides_childs(texel):
        j1 = i1
        j2 = i2
        styles = []
        for child in texel.childs:
            n = length(child)
            if j1 < n and j2 > 0:
                new = get_styles(child, j1, min(j2, n))
                styles = fuse_styles(styles, new)
            j1 = max(0, j1-n)
            j2 = max(0, j2-n)
        return styles
    return [(i2-i1, texel.style)]




def set_styles(texel, i, iterator):
    """Sets the styles from position $i$ on.

       pre:
           is_root_efficient(texel)
       post:
           #out(__return__)
           is_homogeneous(__return__)
           is_list_efficient(__return__)
           length(texel) == calc_length(__return__)
    """

    if texel.is_group:
        r1 = []; r2 = []; r3 = []
        for j1, j2, child in iter_childs(texel):
            if j2 <= i:
                r1.append(child)
            elif iterator.finished:
                r3.append(child)
            else:
                r2 = fuse(r2, set_styles(child, i-j1, iterator))
        return fuse(r1, r2, r3)
    elif texel.is_single:
        if i >= 1:
            return [texel]
        new = texel.set_style(iterator.style)
        iterator.advance(1)
        return [new]
    elif texel.is_text:
        r = []
        text = texel.text
        if i>0:
            r.append(T(text[:i], texel.style))
            text = text[i:]

        while text and not iterator.finished:
            n = min(len(text), iterator.n)
            r.append(T(text[:n], iterator.style))
            iterator.advance(n)
            text = text[n:]
        if text and iterator.finished:
            r.append(T(text, texel.style))
        return r
    elif texel.is_container:
        r1 = []; r2 = []; r3 = []
        for j1, j2, child in iter_childs(texel):
            if j2 <= i:
                r1.append(child)
            elif iterator.finished:
                r3.append(child)
            else:
                r2.append(grouped(set_styles(child, i-j1, iterator)))
        return [texel.set_childs(r1+r2+r3)]
    assert False



def set_properties(texel, i1, i2, properties):
    """Sets text properties in $i1$...$i2$."""
    l1 = get_styles(texel, i1, i2)
    l2 = [(n, updated_style(s, properties)) for n, s in l1]
    return set_styles(texel, i1, StyleIterator(iter(l2)))



def get_parstyles(texel, i1, i2):
    """
    pre:
        0 <= i1 <= i2 <= length(texel)
    post:
        style_length(__return__) == i2-i1
    """
    if i1 == i2:
        return []
    if texel.weights[2]>0 and provides_childs(texel):
        j1 = i1
        j2 = i2
        styles = []
        for child in texel.childs:
            n = length(child)
            if j1 < n and j2 > 0:
                new = get_parstyles(child, j1, min(j2, n))
                styles = fuse_styles(styles, new)
            j1 = max(0, j1-n)
            j2 = max(0, j2-n)
        return styles
    elif isinstance(texel, NewLine):
        return [(i2-i1, texel.parstyle)]
    return [(i2-i1, EMPTYSTYLE)]



def set_parstyles(texel, i, iterator):
    """Sets the paragraph styles from position $i$ on.

       pre:
           is_root_efficient(texel)
       post:
           #out(__return__)
           is_homogeneous(__return__)
           is_list_efficient(__return__)
           length(texel) == calc_length(__return__)
    """
    if not texel.weights[2]:
        iterator.advance(length(texel)-max(0, i))
        return [texel]

    if texel.is_group:
        r1 = []; r2 = []; r3 = []
        for j1, j2, child in iter_childs(texel):
            if j2 <= i:
                r1.append(child)
            elif iterator.finished:
                r3.append(child)
            else:
                r2 = fuse(r2, set_parstyles(child, i-j1, iterator))
        return fuse(r1, r2, r3)
    elif isinstance(texel, NewLine):
        if i >= 1:
            return [texel]
        new = texel.set_parstyle(iterator.style)
        iterator.advance(1)
        return [new]
    elif texel.is_container:
        r1 = []; r2 = []; r3 = []
        for j1, j2, child in iter_childs(texel):
            if j2 <= i:
                r1.append(texel)
            elif iterator.finished:
                r3.append(texel)
            else:
                r2.append(grouped(set_parstyles(child, i-j1, iterator)))
        assert len(r2) == 1
        return texel.set_childs(r1+r2+r3)
    assert False


def set_parproperties(texel, i1, i2, properties):
    """Sets paragraph properties in $i1$...$i2$."""
    l1 = get_parstyles(texel, i1, i2)
    l2 = [(n, updated_style(s, properties)) for n, s in l1]
    return set_parstyles(texel, i1, StyleIterator(iter(l2)))


# -- debug --

def style_length(styles):
    n = 0
    for i, s in styles:
        n += i
    return n


if debug: # enable contract checking
     import contract
     contract.checkmod(__name__)


def test_00():
    "get_style"
    s10 = dict(size=10)
    s12 = dict(size=12)
    g = G([T("01", s10), 
           T("23", s12), 
           T("4567890", s10)])

    #texeltree.dump(g)
    assert get_style(g, 0) is s10
    assert get_style(g, 2) is s12
    assert get_style(g, 4) is s10

def test_01():
    "get_styles"
    s10 = dict(size=10)
    s14 = dict(size=14)
    t = G([T("01234", s10), T("5678", s14)])
    #texeltree.dump(t)
    assert get_styles(t, 0, 2) == [(2, s10)]
    assert get_styles(t, 1, 3) == [(2, s10)]
    assert get_styles(t, 2, 5) == [(3, s10)]
    assert get_styles(t, 0, 9) == [(5, s10), (4, s14)]
    assert get_styles(t, 2, 4) == [(2, s10)]
    assert get_styles(t, 5, 7) == [(2, s14)]


def test_10():
    "set_styles"
    s10 = dict(size=10)
    s12 = dict(size=12)
    s14 = dict(size=14)
    t = T("0123456789", s10)
    styles = [(2, s12), (5, s14)]
    iterator = StyleIterator(iter(styles))
    if 0:
        i = 0
        while not iterator.finished:
            print i, iterator.n, iterator.style
            iterator.advance(1)
            i += 1
    g = grouped(set_styles(t, 2, iterator))
    assert get_pieces(g) == ['01', '23', '45678', '9']

    iterator = StyleIterator(iter([(10, s10)]))
    n = grouped(set_styles(g, 0, iterator))
    assert get_pieces(n) == ['0123456789']

    iterator = StyleIterator(iter([(10, s10)]))
    n = grouped(set_styles(g, 2, iterator))
    assert get_pieces(n) == ['0123456789']

    iterator = StyleIterator(iter([(10, s12)]))
    n = grouped(set_styles(g, 2, iterator))
    assert get_pieces(n) == ['01', '23456789']


def test_11():
    "set_properties"
    s10 = dict(size=10)
    s12 = dict(size=12)
    t = T("0123456789", s10)
    g = grouped(set_properties(t, 2, 4, dict(size=12)))
    #texeltree.dump(g)
    assert get_styles(g, 0, 10) == [(2, s10), (2, s12), (6, s10)]


def test_12():
    "set_parstyles"
    t = grouped([T("0123456789"), NL, T("abcdef"), NL]) 
    iterator = StyleIterator(iter([(10, {'bold':True})]))
    g = grouped(set_parstyles(t, 5, iterator))
    texeltree.dump(g)

    print get_parstyles(g, 0, length(g))




# -*- coding: latin-1 -*-


from .textmodel.textmodel import TextModel, dump_range
from .textmodel.texeltree import Texel, T, G, NL, Container, Single, \
    iter_childs, dump

import wx



def mk_textmodel(texel):
    model = TextModel()
    model.texel = texel
    return model



class Cell(Container):
    pass


class TextCell(Cell):
    text = property(lambda s:s.childs[1])
    def __init__(self, text, **kwargs):
        assert isinstance(text, Texel)
        self.childs = [NL, text, NL]
        self.compute_weights()



class ScriptingCell(Cell):

    client_name = 'direct python'

    input = property(lambda s:s.childs[1])
    output = property(lambda s:s.childs[3])

    def __init__(self, input, output, number = 0, **kwargs):
        assert isinstance(input, Texel)
        assert isinstance(output, Texel)
        self.number = number
        self.childs = [NL, input, NL, output, NL]
        self.compute_weights()




class NotFound(Exception): pass

def find_cell(texel, i, i0=0):
    if isinstance(texel, Cell):
        return i0, texel
    elif texel.is_group:
        for j1, j2, child in iter_childs(texel):
            if j1<=i<j2:
                return find_cell(child, i-j1, i0+j1)
    raise NotFound()



class BitmapRGB(Single):
    def __init__(self, data, size):
        self.data = data
        self.size = size

    def __repr__(self):
        return 'BitmapRGB(...)'



class BitmapRGBA(Single):
    def __init__(self, data, alpha, size):
        self.data = data
        self.alpha = alpha
        self.size = size

    def __repr__(self):
        return 'BitmapRGB(...)'



def test_00():
    "TextCell"
    cell = TextCell(T('0123456789'))
    model = TextModel()
    cellmodel = mk_textmodel(cell)
    model.insert(0, cellmodel)
    dump(model.texel)
    model.insert_text(4, "xyz")
    dump(model.texel)



# -*- coding: latin-1 -*-

#!/usr/bin/env python

from setuptools import setup

# NOTE: if there is a problem with symlinks, update setuptools:
# http://stackoverflow.com/questions/27459888/how-to-make-setuptools-follow-symlinks

with open('README') as file:
    long_description = file.read()

setup(name='pynotebook',
      version='0.1.0',
      description = \
          'A notebook interface similar to mathematica / ipython',
      long_description = long_description,
      author='C. Ecker',
      author_email='textmodelview@gmail.com',
      url='https://pypi.python.org/pypi/pynotebook/',
      license='BSD',
      packages=['pynotebook'],
      platforms = ['any'],
     )




# -*- coding: latin-1 -*-


from .textmodel.textmodel import TextModel
from .textmodel.texeltree import NULL_TEXEL

from .nbtexels import Cell, TextCell, ScriptingCell, mk_textmodel

import re


def get_cells(texel):
    if isinstance(texel, Cell):
        return [texel]
    r = []
    for i1, i2, child in texel.iter_childs():
        r.extend(get_cells(child))
    return r


def totext(model):
    r = []
    for cell in get_cells(model.texel):
        if isinstance(cell, TextCell):
            text = cell.text.get_text()
            r.append("[Text]:\n"+text)
        else:
            text = cell.input.get_text()
            if text:
                r.append(("[In %i]:\n"%cell.number)+text)
            text = cell.output.get_text()
            if text:
                r.append(("[Out %i]:\n"%cell.number)+text)
    return u"\n".join(r)


rx_out = re.compile('\[Out\s*(\w*)\]\:$').match
rx_in = re.compile('\[In\s*(\w*)\]\:$').match
rx_text = re.compile('\[Text\]\:$').match

def fromtext(s, ScriptingCell=ScriptingCell):
    model = TextModel()
    cells = []
    l = []
    fields = [NULL_TEXEL, NULL_TEXEL]
    mode = None
    number = None
    IN = "IN"
    OUT = "OUT"
    TEXT = "TEXT"

    def create_texel():
        texel = TextModel(u'\n'.join(l)).texel
        del l[:]
        return texel

    def flush():
        texel = create_texel()
        if not mode:
            return
        if mode == TEXT:
            cell = TextCell(texel)
        else:
            if mode == IN:
                fields[0] = texel
            else:
                fields[1] = texel
            cell = ScriptingCell(*fields)
            fields[0] = fields[1] = NULL_TEXEL
        cells.append(cell)

    
    for line in s.split('\n'):
        if rx_in(line):
            flush()
            mode = IN
        elif rx_out(line):
            if mode != IN:
                flush()
            else:
                fields[0] = create_texel()
            mode = OUT
        elif rx_text(line):
            flush()
            mode = TEXT
        else:
            l.append(line)
    if l:
        flush()
    model = TextModel()
    for cell in cells:
        model.append(mk_textmodel(cell))
    return model


def test_00():
    text = """[In 1]:
1+2
[Out 2]:
3
[Text]:
Zeile 1
Zeile 2
"""
    m = fromtext(text)
    #m.texel.dump()
    print totext(m)
    t = totext(m)
    print repr(text)
    print repr(t)
    assert t == text
